@@ -4,7 +4,7 @@
 
 import itertools
 import unittest
-from typing import ClassVar, Collection
+from typing import ClassVar, Collection, Optional
 
 import click
 from click.testing import CliRunner, Result
@@ -20,7 +20,7 @@ except ImportError:
 class Base:
     """A base class."""
 
-    def __init__(self, name):
+    def __init__(self, name: str):
         """Initialize the class."""
         self.name = name
 
@@ -47,6 +47,14 @@ class D(Base):
     """D base class."""
 
 
+class E(Base):
+    """E base class."""
+
+    def __init__(self, name: Optional[str] = None):
+        """Initialize the class."""
+        super().__init__(name or "default_name")
+
+
 class AltBase:
     """An alternative base class."""
 
@@ -60,7 +68,7 @@ class TestResolver(unittest.TestCase):
 
     def setUp(self) -> None:
         """Set up the resolver class."""
-        self.resolver = Resolver([A, B, C], base=Base)
+        self.resolver = Resolver([A, B, C, E], base=Base)
 
     def test_contents(self):
         """Test the functions."""
@@ -68,7 +76,7 @@ class TestResolver(unittest.TestCase):
 
     def test_iterator(self):
         """Test iterating over classes."""
-        self.assertEqual([A, B, C], list(self.resolver))
+        self.assertEqual([A, B, C, E], list(self.resolver))
 
     def test_lookup(self):
         """Test looking up classes."""
@@ -209,3 +217,62 @@ class TestResolver(unittest.TestCase):
         with self.assertRaises(UnexpectedKeywordError) as e:
             resolver.make("A", nope="nopppeeee")
             self.assertEqual("AAltBase did not expect any keyword arguments", str(e))
+
+    def test_make_many(self):
+        """Test the make_many function."""
+        with self.assertRaises(ValueError):
+            # no default is given
+            self.resolver.make_many(None)
+
+        with self.assertRaises(ValueError):
+            # wrong number of kwargs is given
+            self.resolver.make_many([], [{}, {}])
+
+        with self.assertRaises(ValueError):
+            # wrong number of kwargs is given
+            self.resolver.make_many(["a", "a", "a"], [{}, {}])
+
+        # One class, one kwarg
+        instances = self.resolver.make_many("a", dict(name="name"))
+        self.assertEqual([A(name="name")], instances)
+        instances = self.resolver.make_many("a", [dict(name="name")])
+        self.assertEqual([A(name="name")], instances)
+        instances = self.resolver.make_many(["a"], dict(name="name"))
+        self.assertEqual([A(name="name")], instances)
+        instances = self.resolver.make_many(["a"], [dict(name="name")])
+        self.assertEqual([A(name="name")], instances)
+
+        # Single class, multiple kwargs
+        instances = self.resolver.make_many("a", [dict(name="name1"), dict(name="name2")])
+        self.assertEqual([A(name="name1"), A(name="name2")], instances)
+        instances = self.resolver.make_many(["a"], [dict(name="name1"), dict(name="name2")])
+        self.assertEqual([A(name="name1"), A(name="name2")], instances)
+
+        # Multiple class, one kwargs
+        instances = self.resolver.make_many(["a", "b", "c"], dict(name="name"))
+        self.assertEqual([A(name="name"), B(name="name"), C(name="name")], instances)
+        instances = self.resolver.make_many(["a", "b", "c"], [dict(name="name")])
+        self.assertEqual([A(name="name"), B(name="name"), C(name="name")], instances)
+
+        # Multiple class, multiple kwargs
+        instances = self.resolver.make_many(
+            ["a", "b", "c"], [dict(name="name1"), dict(name="name2"), dict(name="name3")]
+        )
+        self.assertEqual([A(name="name1"), B(name="name2"), C(name="name3")], instances)
+
+        # One class, No kwargs
+        instances = self.resolver.make_many("e")
+        self.assertEqual([E()], instances)
+        instances = self.resolver.make_many(["e"])
+        self.assertEqual([E()], instances)
+        instances = self.resolver.make_many("e", None)
+        self.assertEqual([E()], instances)
+        instances = self.resolver.make_many(["e"], None)
+        self.assertEqual([E()], instances)
+        instances = self.resolver.make_many(["e"], [None])
+        self.assertEqual([E()], instances)
+
+        # No class
+        resolver = Resolver.from_subclasses(Base, default=A)
+        instances = resolver.make_many(None, dict(name="name"))
+        self.assertEqual([A(name="name")], instances)
